<?php
/*
	Copyright 2022 EvolCorr

	This file is part of Brandly Reseller

	Brandly Reseller is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class BrandlyCoZaResellerApi {
	
	protected const BRANDLY_BASE_URL = "https://brandly.co.za";
	protected $brandlyApiToken;
	
	
	function __construct(string $token) {
		$this->brandlyApiToken = $token;
	}
	

	protected static function _apiCall(string $url, array $additionalHeaders = [], string $method="GET", string $data=''){		

		$header = "Content-Type: application/json\r\nContent-Length: " . strlen($data);
		foreach($additionalHeaders as $h) $header .= "\r\n$h";	
		$options = ['http' => [ 'method'  => $method, 'content' => $data, 'header'  => $header, 'ignore_errors' =>true ], 'ssl' => ['verify_peer' => false ] ];
		$context  = stream_context_create($options);	
		$result = file_get_contents($url, false, $context);	
			
		if(!strpos($result,"results") && !strpos($result,"token")){
			$error = error_get_last();
			$respHeaders = print_r($http_response_header,true);
			if(strpos($result,"incorrect_password") !== false) $log = "⚠️ Incorrect username or password";
			else if(strpos($result,"ip_blocked") !== false) $log = "⚠️ You have exceeded maximum retries. Please try after 15 minutes.";
			else $log = "<br>\r\n$url: HTTP $method request failed. Resp headers: $respHeaders . Error: " . ($error['message'] ?? "") . "<br>\r\ndata:$data<br>\r\nresult:$result<br>\r\n";
			//echo $log;
			throw new \Exception($log);
		}
		
		return json_decode($result);
		
	}


	public function apiCall(string $apiCall, string $httpMethod="GET", string $body=''){	
		$response = self::_apiCall(self::BRANDLY_BASE_URL . "/wp-json/brandly-api/v1/$apiCall",["Authorization: Bearer $this->brandlyApiToken"],$httpMethod,$body);	
		if(!isset($response->results)){ 
			$message = "";
			if($response && !is_string($response)) foreach($response as $key => $value) $message .= "$key: " . (is_string($value) ? $value : print_r($value,true)) . " <br>";
			else if(!$response) throw new \Exception("No response from apicall $httpMethod $apiCall");
			else $message = "Response from apiCall $httpMethod $apiCall: $response";
			throw new \Exception($message);
		}
		return $response;
	}


	public function getCategories(){	
		return $this->apiCall("categories");
	}
	
	public function getRandRates(string $domain){	
		return $this->apiCall("get-rand-rates?domain=$domain");
	}
	
	public function createProductSearch($limit=5) : object {	
		return json_decode(json_encode(['ids'=>[],'categoryId'=>null, 'sku'=>null, 's'=>'', 'offset'=>0, 'limit'=>$limit]));
	}
	
	public function productSearchAddCategoryId(object $params, int $categoryId){	
		$params->categoryId=$categoryId;
	}
	
	public function productSearchAddId(object $params, int $id){	
		$params->ids[]=$id;
	}
	
	public function productSearchAddSku(object $params, string $sku){	
		$params->sku=$sku;
	}
	
	public function productSearchAddTerms(object $params, string $terms){	
		$params->s=$terms;
	}
	
	public function productSearchLimit(object $params, int $limit){	
		$params->limit=$limit;
	}
	
	public function productSearchOffset(object $params, int $offset){	
		$params->offset=$offset;
	}
	
	public function getProducts(object $searchParams){
		$call = "search?";
		if($searchParams->ids && is_array($searchParams->ids)){
			$areAllNumeric = true;
			foreach($searchParams->ids as $id){
				if(!is_numeric($id)){
					$areAllNumeric = false;
					break;
				}
			}
			if($areAllNumeric) $call .= "idIn=" . implode(",",$searchParams->ids) . "&";
		}
		if($searchParams->categoryId){
			$searchParams->categoryId = max(0,intval($searchParams->categoryId));
			if($searchParams->categoryId) $call .= "categoryId=$searchParams->categoryId&";
		}
		if($searchParams->sku){ 
			$searchParams->sku = trim(preg_replace('!\s+!', ' ', $searchParams->sku));
			if($searchParams->sku) $call .= "sku=$searchParams->sku&";
		}
		if($searchParams->s) $call .= "s=" . trim($searchParams->s) . "&";
		$searchParams->offset=max(0,intval($searchParams->offset));
		$searchParams->limit=max(1,intval($searchParams->limit));
		$call .= "offset=$searchParams->offset&limit=$searchParams->limit";
		return $this->apiCall($call);
	}
	
	
	public static function convertPriceFromZAR(float $price,object $priceRates,string $storeCurrencyCode){ // convert price from ZAR to store currency
		if(!$price) return $price;
		return sprintf("%.02f",$price / $priceRates->$storeCurrencyCode);
	}

}


